@@ -57,6 +57,12 @@
 
 #include "gdm-socket-protocol.h"
 
+#if __sun
+#define GDM_PRIO_DEFAULT NZERO
+#else
+#define GDM_PRIO_DEFAULT 0
+#endif
+
 /* Local prototypes */
 static void gdm_server_spawn (GdmDisplay *d, const char *vtarg);
 static void gdm_server_usr1_handler (gint);
@@ -137,122 +143,32 @@ jumpback_xioerror_handler (Display *disp)
 static void
 gdm_exec_fbconsole (GdmDisplay *disp)
 {
-        pid_t pid;
         char *argv[6];
 
         argv[0] = FBCONSOLE;
-        argv[1] = "-d";
-        argv[2] = disp->name;
-        argv[3] = NULL;
+        argv[1] = "-n";
+        argv[2] = "-d";
+        argv[3] = disp->name;
+        argv[4] = NULL;
 
-	g_debug ("Forking fbconsole");
+	gdm_debug ("Forking fbconsole");
 
-        pid = fork ();
-        if (pid == 0) {
+        d->fbconsolepid = fork ();
+        if (d->fbconsolepid == 0) {
                 gdm_close_all_descriptors (0 /* from */, -1 /* except */, -1 /* except2 */)
 ;
                 VE_IGNORE_EINTR (execv (argv[0], argv));
+
+		gdm_error ("Can not start fallback console: %s",
+			   strerror (errno));
+		_exit (0);
         }
-        if (pid == -1) {
+        if (d->fbconsolepid == -1) {
                 gdm_error (_("Can not start fallback console"));
         }
 }
 #endif
 
-/**
- * gdm_server_reinit:
- * @disp: Pointer to a GdmDisplay structure
- *
- * Reinit the display, basically sends a HUP signal
- * but only if the display exists
- */
-
-gboolean
-gdm_server_reinit (GdmDisplay *disp)
-{
-	if (disp == NULL)
-		return FALSE;
-
-	if (disp->servpid <= 0) {
-		/* Kill our connection if one existed, likely to result
-		 * in some bizzaro error right now */
-		if (disp->dsp != NULL) {
-			XCloseDisplay (disp->dsp);
-			disp->dsp = NULL;
-		}
-		return FALSE;
-	}
-
-	gdm_debug ("gdm_server_reinit: Server for %s is about to be reinitialized!", disp->name);
-
-	if ( ! setup_server_wait (disp))
-		return FALSE;
-
-	d->servstat = SERVER_PENDING;
-
-	if (disp->dsp != NULL) {
-		/* static because of the Setjmp */
-		static int (*old_xerror_handler)(Display *, XErrorEvent *) = NULL;
-		static int (*old_xioerror_handler)(Display *) = NULL;
-
-		old_xerror_handler = NULL;
-		old_xioerror_handler = NULL;
-
-		/* Do note the interaction of this Setjmp and the signal
-	   	   handlers and the Setjmp in slave.c */
-
-		/* Long live Setjmp, DIE DIE DIE XSetIOErrorHandler */
-
-		if (Setjmp (reinitjmp) == 0)  {
-			/* come here and we'll whack the server and wait to get
-			   an xio error */
-			old_xerror_handler = XSetErrorHandler (ignore_xerror_handler);
-			old_xioerror_handler = XSetIOErrorHandler (jumpback_xioerror_handler);
-
-			/* Now whack the server with a SIGHUP */
-			gdm_sigchld_block_push ();
-			if (disp->servpid > 1)
-				kill (disp->servpid, SIGHUP);
-			else
-				d->servstat = SERVER_DEAD;
-			gdm_sigchld_block_pop ();
-
-			/* the server is dead, weird */
-			if (disp->dsp != NULL) {
-				XCloseDisplay (disp->dsp);
-				disp->dsp = NULL;
-			}
-		}
-		/* no more display */
-		disp->dsp = NULL;
-		XSetErrorHandler (old_xerror_handler);
-		XSetIOErrorHandler (old_xioerror_handler);
-	} else {
-		/* Now whack the server with a SIGHUP */
-		gdm_sigchld_block_push ();
-		if (disp->servpid > 1)
-			kill (disp->servpid, SIGHUP);
-		else
-			d->servstat = SERVER_DEAD;
-		gdm_sigchld_block_pop ();
-	}
-
-	/* Wait for the SIGUSR1 */
-	do_server_wait (d);
-
-	if (d->servstat == SERVER_RUNNING) {
-#ifdef HAVE_FBCONSOLE
-		gdm_exec_fbconsole (d);
-#endif
-		return TRUE;
-        } else {
-		/* if something really REALLY screwed up, then whack the
-		   lockfiles for safety */
-		gdm_server_whack_lockfile (d);
-		return FALSE;
-	}
-}
-
 /**
  * gdm_server_stop:
  * @disp: Pointer to a GdmDisplay structure
@@ -335,6 +251,13 @@ gdm_server_stop (GdmDisplay *disp)
 
     gdm_server_wipe_cookies (disp);
 
+#ifdef HAVE_FBCONSOLE
+    /* Kill fbconsole if it is running */
+    if (d->fbconsolepid > 0)
+        kill (d->fbconsolepid, SIGTERM);
+    d->fbconsolepid = 0;
+#endif
+
     gdm_slave_whack_temp_auth_file ();
 }
 
@@ -731,6 +654,10 @@ gdm_server_start (GdmDisplay *disp,
 
     d = disp;
 
+#ifdef HAVE_FBCONSOLE
+    d->fbconsolepid = 0;
+#endif
+
     /* if an X server exists, wipe it */
     gdm_server_stop (d);
 
@@ -780,6 +707,8 @@ gdm_server_start (GdmDisplay *disp,
     /* fork X server process */
     gdm_server_spawn (d, vtarg);
 
+    g_free (vtarg);
+
     /* we can now use d->handled since that's set up above */
     do_server_wait (d);
 
@@ -1045,15 +974,25 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 			g_shell_parse_argv (str, &argc, &argv, NULL);
 
 		} else {
+			GError* error_p;
 			char **svr_command;
 			const char *str;
 			int svr_argc;
 
 			str = ve_sure_string (svr->command);
 			svr_command = NULL;
-			g_shell_parse_argv (str, &svr_argc, &svr_command, NULL);
 
-			g_shell_parse_argv (disp->command, &argc, &argv, NULL);
+			g_shell_parse_argv (str, &svr_argc,
+				&svr_command, &error_p);
+
+			g_shell_parse_argv (disp->command, &argc,
+				&argv, &error_p);
+
+			if (argv == NULL) {
+				gdm_debug ("Problem parsing server command <%s>",
+					disp->command ? disp->command : "(null)");
+				return FALSE;
+			}
 
 			if (argv[0] == NULL || argv[1] == NULL) {
 				g_strfreev (argv);
@@ -1119,7 +1058,7 @@ gdm_server_resolve_command_line (GdmDisplay *disp,
 		disp->handled = FALSE;
 		/* never ever ever use chooser here */
 		disp->use_chooser = FALSE;
-		disp->priority = 0;
+		disp->priority = GDM_PRIO_DEFAULT;
 		/* run just one session */
 		argv[len++] = g_strdup ("-terminate");
 		argv[len++] = g_strdup ("-query");
@@ -1169,6 +1108,7 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
     int logfd;
     char *command;
     pid_t pid;
+    gboolean rc;
 
     if (d == NULL ||
 	ve_string_empty (d->command)) {
@@ -1191,11 +1131,13 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
     /* Figure out the server command */
     argv = NULL;
     argc = 0;
-    gdm_server_resolve_command_line (d,
-				     TRUE /* resolve flags */,
-				     vtarg,
-				     &argc,
-				     &argv);
+    rc = gdm_server_resolve_command_line (d,
+				         TRUE /* resolve flags */,
+				         vtarg,
+				         &argc,
+				         &argv);
+    if (rc == FALSE)
+       return;
 
     /* Do not support additional session arguments with Xnest. */
     if (d->type != TYPE_FLEXI_XNEST) {
@@ -1208,7 +1150,7 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
     /* Fork into two processes. Parent remains the gdm process. Child
      * becomes the X server. */
 
-    g_debug ("Forking X server process");
+    gdm_debug ("Forking X server process");
 
     gdm_sigterm_block_push ();
     pid = d->servpid = fork ();
@@ -1253,6 +1195,8 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 			   "gdm_server_spawn", d->name);
 	}
 
+	g_free (logfile);
+
 	/* The X server expects USR1/TTIN/TTOU to be SIG_IGN */
 	ign_signal.sa_handler = SIG_IGN;
 	ign_signal.sa_flags = SA_RESTART;
@@ -1338,7 +1282,7 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 
 	gdm_debug ("gdm_server_spawn: '%s'", command);
 	
-	if (d->priority != 0) {
+	if (d->priority != GDM_PRIO_DEFAULT) {
 		if (setpriority (PRIO_PROCESS, 0, d->priority)) {
 			gdm_error (_("%s: Server priority couldn't be set to %d: %s"),
 				   "gdm_server_spawn", d->priority,
@@ -1395,12 +1339,12 @@ gdm_server_spawn (GdmDisplay *d, const char *vtarg)
 		setgroups (1, groups);
 	}
 
-#if sun
+#if __sun
     {
         /* Remove old communication pipe, if present */
         char old_pipe[MAXPATHLEN];
 
-        sprintf (old_pipe, "%s/%d", SDTLOGIN_DIR, d->name);
+        sprintf (old_pipe, "%s/%d", GDM_SDTLOGIN_DIR, d->name);
         g_unlink (old_pipe);
     }
 #endif
